@@ -1,25 +1,24 @@
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
 /*
-    Copyright (C) 2012-15  Vishesh Handa <vhanda@kde.org>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+ *    Copyright (C) 2012-15  Vishesh Handa <vhanda@kde.org>
+ * 
+ *    This library is free software; you can redistribute it and/or
+ *    modify it under the terms of the GNU Lesser General Public
+ *    License as published by the Free Software Foundation; either
+ *    version 2.1 of the License, or (at your option) any later version.
+ * 
+ *    This library is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *    Lesser General Public License for more details.
+ * 
+ *    You should have received a copy of the GNU Lesser General Public
+ *    License along with this library; if not, write to the Free Software
+ *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #include "exiv2extractor.h"
-// #include "exiv2.hpp"
 
 #include <QGeoAddress>
 #include <QFileInfo>
@@ -28,10 +27,13 @@
 #include <QFile>
 #include <QTextCodec>
 
+#include "geolocation/cities.h"
+#include "geolocation/city.h"
+
 Exiv2Extractor::Exiv2Extractor(const QUrl &url, QObject *parent) : QObject(parent)
-    , m_error(true)
-    , m_url(url)
-    {
+, m_error(true)
+, m_url(url)
+{
     if (!QFileInfo::exists(m_url.toLocalFile()) || m_url.isEmpty() || !m_url.isValid()) {
         m_error = true;
     }    
@@ -48,13 +50,13 @@ Exiv2Extractor::Exiv2Extractor(const QUrl &url, QObject *parent) : QObject(paren
     if (!m_image->good()) {
         return;
     }
-        
+    
     try {
         m_image->readMetadata();
     } catch (const std::exception &) {
         return;
     }
-        
+    
     m_error = false;
 }
 
@@ -63,7 +65,7 @@ Exiv2::ExifData & Exiv2Extractor::exifData() const
 {   
     Exiv2::ExifData &exifData = m_image->exifData();
     if (exifData.empty()) {
-        qWarning() <<  "No EXIF data in : " << m_url.toString();
+//        qWarning() <<  "No EXIF data in : " << m_url.toString();
     }    
     
     return exifData;
@@ -71,13 +73,13 @@ Exiv2::ExifData & Exiv2Extractor::exifData() const
 
 Coordinates Exiv2Extractor::extractGPS() const
 {
-   double latitude = fetchGpsDouble("Exif.GPSInfo.GPSLatitude");
-   double longitude = fetchGpsDouble("Exif.GPSInfo.GPSLongitude");
-
+    double latitude = fetchGpsDouble("Exif.GPSInfo.GPSLatitude");
+    double longitude = fetchGpsDouble("Exif.GPSInfo.GPSLongitude");
+    
     QByteArray latRef = getExifTagData("Exif.GPSInfo.GPSLatitudeRef");
     if (!latRef.isEmpty() && latRef[0] == 'S')
         latitude *= -1;
-
+    
     QByteArray longRef = getExifTagData("Exif.GPSInfo.GPSLongitudeRef");
     if (!longRef.isEmpty() && longRef[0] == 'W')
         longitude *= -1;
@@ -92,43 +94,43 @@ double Exiv2Extractor::fetchGpsDouble(const char *name) const
     if (it != data.end() && it->count() == 3) {
         double n = 0.0;
         double d = 0.0;
-
+        
         n = (*it).toRational(0).first;
         d = (*it).toRational(0).second;
-
+        
         if (d == 0) {
             return 0.0;
         }
-
+        
         double deg = n / d;
-
+        
         n = (*it).toRational(1).first;
         d = (*it).toRational(1).second;
-
+        
         if (d == 0) {
             return deg;
         }
-
+        
         double min = n / d;
         if (min != -1.0) {
             deg += min / 60.0;
         }
-
+        
         n = (*it).toRational(2).first;
         d = (*it).toRational(2).second;
-
+        
         if (d == 0) {
             return deg;
         }
-
+        
         double sec = n / d;
         if (sec != -1.0) {
             deg += sec / 3600.0;
         }
-
+        
         return deg;
     }
-
+    
     return 0.0;
 }
 
@@ -160,7 +162,7 @@ QString Exiv2Extractor::getExifTagString(const char* exifTagName, bool escapeCR)
     }
     catch( Exiv2::Error& e )
     {
-        qWarning() << (QString::fromLatin1("Cannot find Exif key '%1' into image using Exiv2 ").arg(QString::fromLatin1(exifTagName)), e.what());
+        qWarning() << QString("Cannot find Exif key '%1' into image using Exiv2 ").arg(QString::fromLatin1(exifTagName)) << e.what();
     }
     catch(...)
     {
@@ -190,7 +192,7 @@ QByteArray Exiv2Extractor::getExifTagData(const char* exifTagName) const
     }
     catch( Exiv2::Error& e )
     {
-        qWarning() << (QString::fromLatin1("Cannot find Exif key '%1' into image using Exiv2 ").arg(QString::fromLatin1(exifTagName)), e.what());
+        qWarning() << QString("Cannot find Exif key '%1' into image using Exiv2 ").arg(QString::fromLatin1(exifTagName)) << e.what();
     }
     catch(...)
     {
@@ -275,11 +277,11 @@ QVariant Exiv2Extractor::getExifTagVariant(const char* exifTagName, bool rationa
     }
     catch( Exiv2::Error& e )
     {
-       qWarning () << (QString::fromLatin1("Cannot find Exif key '%1' in the image using Exiv2 ").arg(QString::fromLatin1(exifTagName)), e.what());
+        qWarning () << QString("Cannot find Exif key '%1' in the image using Exiv2 ").arg(QString::fromLatin1(exifTagName)) << e.what();
     }
     catch(...)
     {
-       qWarning() << "Default exception from Exiv2";
+        qWarning() << "Default exception from Exiv2";
     }
     
     return QVariant();
@@ -288,7 +290,7 @@ QVariant Exiv2Extractor::getExifTagVariant(const char* exifTagName, bool rationa
 static bool isUtf8(const char* const buffer)
 {
     int i, n;
-     unsigned char c;
+    unsigned char c;
     bool gotone = false;
     
     if (!buffer)
@@ -490,7 +492,6 @@ MetaDataMap Exiv2Extractor::getExifTagsDataList(const QStringList& exifKeysFilte
         Exiv2::ExifData &data = exifData();
         data.sortByKey();
         
-        QString     ifDItemName;
         MetaDataMap metaDataMap;
         
         for (Exiv2::ExifData::iterator md = data.begin(); md != data.end(); ++md)
@@ -544,11 +545,11 @@ MetaDataMap Exiv2Extractor::getExifTagsDataList(const QStringList& exifKeysFilte
     }
     catch (Exiv2::Error& e)
     {
-       qWarning() << (QString::fromLatin1("Cannot parse EXIF metadata using Exiv2 "), e.what());
+        qWarning() << (QString::fromLatin1("Cannot parse EXIF metadata using Exiv2 "), e.what());
     }
     catch(...)
     {
-       qWarning() << "Default exception from Exiv2";
+        qWarning() << "Default exception from Exiv2";
     }
     
     return MetaDataMap();
@@ -606,39 +607,52 @@ QString Exiv2Extractor::getExifComment() const
     return QString();
 }
 
-QString Exiv2Extractor::GPSString()
+QString Exiv2Extractor::GPSString() const
 {
-    auto coordinates = extractGPS();
+    if(error())
+    {
+        return QString();
+    }
+     auto m_city = city();
     
-    auto latitude = coordinates.first;
-    auto longitude = coordinates.second;
-    if (latitude != 0.0 && longitude != 0.0) {
-        QVariantMap map = m_geoCoder.lookup(latitude, longitude);
-        
-        QGeoAddress addr;
-        addr.setCountry(map.value("country").toString());
-        addr.setState(map.value("admin1").toString());
-        addr.setCity(map.value("admin2").toString());
-       return addr.text();
+    if(!m_city.isValid())
+    {
+        return QString();
     }
     
-    //    double latitude = extractor.gpsLatitude();
-    //    double longitude = extractor.gpsLongitude();
+    return m_city.name();
+}
+
+QString Exiv2Extractor::cityId() const
+{
+    if(error())
+    {
+        return QString();
+    }
     
-    //    if (latitude != 0.0 && longitude != 0.0)
-    //    {
-    //        if (!m_geoCoder->initialized())
-    //        {
-    //            m_geoCoder->init();
-    //        }
-    //        QVariantMap map = m_geoCoder->lookup(latitude, longitude);
+    auto m_city = city();
     
-    //        QGeoAddress addr;
-    //        addr.setCountry(map.value("country").toString());
-    //        addr.setState(map.value("admin1").toString());
-    //        addr.setCity(map.value("admin2").toString());
-    //        m_data << FMH::MODEL{{FMH::MODEL_KEY::KEY, "Location"}, {FMH::MODEL_KEY::VALUE, addr.text()}};
-    //    }
-    //    m_data << FMH::MODEL{{FMH::MODEL_KEY::KEY, "Origin"}, {FMH::MODEL_KEY::VALUE, extractor.dateTime().toString()}};
-    return QString();
+    if(!m_city.isValid())
+    {
+        return QString();
+    }
+    
+    return m_city.id();
+}
+
+const City Exiv2Extractor::city() const
+{
+    if(error())
+    {
+        return City();
+    }
+    
+    auto c = extractGPS();
+    
+    if(c.first == 0.0 || c.second == 0.0)
+    {
+        return City();
+    }
+    
+    return Cities::getInstance()->findCity(c.first, c.second);
 }
